 SUBROUTINE user_init_grid( gls, nzb_local )

!------------------------------------------------------------------------------!
! Current revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Id: user_init_grid.f90 392 2009-09-24 10:39:14Z raasch $
!
! 258 2009-03-13 12:36:03Z heinze
! Output of messages replaced by message handling routine.
! add 'single_street_canyon' as standard topography case
! -topography_grid_convention
!
! 217 2008-12-09 18:00:48Z letzel
! +topography_grid_convention
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
! Execution of user-defined grid initializing actions
! First argument gls contains the number of ghost layers, which is > 1 if the
! multigrid method for the pressure solver is used
!------------------------------------------------------------------------------!

    USE control_parameters
    USE indices
    USE user

    IMPLICIT NONE

    INTEGER ::  gls

    INTEGER, DIMENSION(-gls:ny+gls,-gls:nx+gls) ::  nzb_local

!
!-- Here the user-defined grid initializing actions follow:

!
!-- Set the index array nzb_local for non-flat topography.
!-- Here consistency checks concerning domain size and periodicity are necessary
    SELECT CASE ( TRIM( topography ) )

       CASE ( 'flat', 'single_building', 'single_street_canyon' )
!
!--       Not allowed here since these are the standard cases used in init_grid.

       CASE ( 'user_defined_topography_1' )
!
!--       Here the user can define his own topography.
!--       After definition, please remove the following three lines!
          message_string = 'topography "' // topography // '" not available yet'
          CALL message( 'user_init_grid', 'UI0005', 1, 2, 0, 6, 0 )

       CASE DEFAULT
!
!--       The DEFAULT case is reached if the parameter topography contains a
!--       wrong character string that is neither recognized in init_grid nor 
!--       here in user_init_grid.
          message_string = 'unknown topography "' // topography // '"'
          CALL message( 'user_init_grid', 'UI0006', 1, 2, 0, 6, 0 )

    END SELECT


 END SUBROUTINE user_init_grid

